#!/bin/sh

set -e -o pipefail

# Init Java options - ORG_JAVA_OPTS will keep the original value when the script was started for the first time
JAVA_OPTS="${ORG_JAVA_OPTS:=${JAVA_OPTS:- }}"
export ORG_JAVA_OPTS

# Allow custom props to be inserted
#test -f /custom-conf1.sh && . /custom-conf1.sh

# General Java options
JAVA_OPTS+=" -showversion"
JAVA_OPTS+=" -XX:+PrintFlagsFinal"
JAVA_OPTS+=" -XX:+PrintCommandLineFlags"
JAVA_OPTS+=" -XX:+PrintStringTableStatistics"
JAVA_OPTS+=" -XX:ErrorFile=/tmp/error_java_identity_%p.log"
JAVA_OPTS+=" -Xlog:gc*:/tmp/identity.txt:time,level,tags:filesize=50M,filecount=1"

# memory options
# use parallel GC if no GC is added in JAVA_OPTS - the default G1 uses too much off-heap memory
if ! grep -E 'UseG1GC|UseConcMarkSweepGC|UseEpsilonGC|UseParallelGC|UseSerialGC|UseZGC' <<< "$JAVA_OPTS" > /dev/null
then
  echo "Setting GC to ParallelGC"
  JAVA_OPTS+=" -XX:+UseParallelGC"
fi

JAVA_OPTS+=" -Xms${JVM_XMS:-768m} -Xmx${JVM_XMX:-1024m}"

echo "Setting metaspace to ${JVM_METASPACE:=256m}"
JAVA_OPTS+=" -XX:MaxMetaspaceSize=$JVM_METASPACE"

JAVA_OPTS+=" -Xss384k"
JAVA_OPTS+=" -XX:MaxJavaStackTraceDepth=0"
JAVA_OPTS+=" -XX:SurvivorRatio=8"
JAVA_OPTS+=" -XX:TargetSurvivorRatio=90"
JAVA_OPTS+=" -XX:MaxTenuringThreshold=15"
JAVA_OPTS+=" -XX:ParallelGCThreads=4"
JAVA_OPTS+=" -XX:+OptimizeStringConcat"

# https://jira.eng.vmware.com/browse/VEBS-340
HEAP_DUMP_PATH="/heapDump/$( date '+%Y-%m-%d_%H-%M-%S' )_heap_dump.hprof"
JAVA_OPTS+=" -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=$HEAP_DUMP_PATH"
JAVA_OPTS+=" -XX:+ExitOnOutOfMemoryError"

# Allows for changing the restricted headers (such as 'Host') when using RestTemplate
# https://stackoverflow.com/questions/43223261/setting-host-header-for-spring-resttemplate-doesnt-work
JAVA_OPTS+=" -Dsun.net.http.allowRestrictedHeaders=true"

if [ -n "$DEBUG_PORT" ]; then
  echo "Enabling debugging on port: $DEBUG_PORT"
  JAVA_OPTS+=" -agentlib:jdwp=transport=dt_socket,server=y,suspend=y,address=*:$DEBUG_PORT"
fi

#
# Add ingress/load-balancert/proxy certificates in the java keystore if such are mounted,
# configure FIPS properties
#
#source /usr/local/bin/setup_security.sh

if [ -n "$SERVER_PORT" ]; then
  JAVA_OPTS+=" -Dserver.port=$SERVER_PORT"
fi

# Configure Datasource
DATASOURCE_URI=""
if [ -n "$DATASOURCE_URL" ]; then
  DATASOURCE_URI=$DATASOURCE_URL
fi

if [ -z "$DATASOURCE_URI" -a -n "$PGHOST" -a -n "$PGDATABASE" ]; then
  DATASOURCE_URI="jdbc:postgresql://$PGHOST"
  if [ -n "$PGPORT" ]; then
    DATASOURCE_URI+=":$PGPORT"
  fi

  DATASOURCE_URI+="/$PGDATABASE"
fi

if [ -n "$DATASOURCE_URI" ]; then
  JAVA_OPTS+=" -Dspring.datasource.url=$DATASOURCE_URI"
  if [ -n "$DATASOURCE_PARAMS" ]; then
   JAVA_OPTS+="?$DATASOURCE_PARAMS"
  fi
fi

if [ -n "$DATASOURCE_USER" ]; then
  JAVA_OPTS+=" -Dspring.datasource.username=$DATASOURCE_USER"
fi

# Init Spring profiles string
SPRING_PROFILES=" -Dspring.profiles.active="


# Append any profiles that are configured other than csp
if [ -n "$ADDITIONAL_SPRING_PROFILES" ]; then
  SPRING_PROFILES+="$ADDITIONAL_SPRING_PROFILES"
fi

# Always setting csp profile until below ticket is closed
# TODO: https://jira.carbonblack.local/browse/CBC-30633

#if [ -n "$CSP_URI" ]; then
if [ -n "$ADDITIONAL_SPRING_PROFILES" ]; then
  SPRING_PROFILES+=",csp"
else
  SPRING_PROFILES+="csp"
fi
#fi

JAVA_OPTS+="$SPRING_PROFILES"
JAVA_OPTS+=" -XX:-OmitStackTraceInFastThrow"

# Configure CSP
if [ -n "$CSP_URI" ]; then
  JAVA_OPTS+=" -Dcbc.identity.csp.uri=$CSP_URI"
  JAVA_OPTS+=" -Dcbc.identity.csp.client-id=$CSP_AUTH_CLIENT_ID"
  JAVA_OPTS+=" -Dcbc.identity.csp.client-secret=$CSP_AUTH_CLIENT_SECRET"
  JAVA_OPTS+=" -Dcbc.identity.csp.service-id=$CSP_SERVICE_ID"
  JAVA_OPTS+=" -Dcbc.identity.csp.service-instance-id=$CSP_SERVICE_INSTANCE_ID"
  JAVA_OPTS+=" -Dcbc.identity.csp.service-instance-id-csr=$CSP_SERVICE_INSTANCE_ID_CSR"
  JAVA_OPTS+=" -Dcbc.identity.csp.operator-org-id=$OPERATOR_ORG_ID"
  
  if [ -n "$CSP_SYNC" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.csp-sync-enabled=$CSP_SYNC"
  fi
  if [ -n "$CSP_ORG_SYNC" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.csp-org-sync-enabled=$CSP_ORG_SYNC"
  fi
  if [ -n "$CSP_OP_ORG_SYNC" ]; then
      JAVA_OPTS+=" -Dcbc.identity.backend.csp-op-org-sync-enabled=$CSP_OP_ORG_SYNC"
  fi
  if [ -n "$CSP_DELETE_ORG_SYNC" ]; then
      JAVA_OPTS+=" -Dcbc.identity.backend.csp-org-sync-delete-enabled=$CSP_DELETE_ORG_SYNC"
  fi
  if [ -n "$CSP_NOTIFICATIONS" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.csp-notifications=$CSP_NOTIFICATIONS"
  fi
fi

if [ -n "$ADMIN_INSTANCE" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.admin-instance=$ADMIN_INSTANCE"
fi

if [ -n "$CLM_NOTIFICATIONS_MESSAGE_TYPE" ]; then
    JAVA_OPTS+=" -Dnotifications.clm.csp-message-types.customer-lifecycle-event=$CLM_NOTIFICATIONS_MESSAGE_TYPE"
fi

if [ -n "$DEFENSE_BACKEND_URI" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.defense.app-service-uri=$DEFENSE_BACKEND_URI"
fi

if [ -n "$INTERNAL_GATEWAY_URL" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.defense.internal-gateway-uri=$INTERNAL_GATEWAY_URL"
fi

if [ -n "$EXTERNAL_GATEWAY_URL" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.defense.external-gateway-uri=$EXTERNAL_GATEWAY_URL"
fi

if [ -n "$PSC_GATEWAY_HOST" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.defense.psc-gateway-host=$PSC_GATEWAY_HOST"
fi

if [ -n "$PREVENT_CSR_TO_MANAGE_CUSTOMER_CONNECTORS" ]; then
  JAVA_OPTS+=" -Dcbc.identity.backend.defense.prevent-csr-users-to-manage-customer-connectors=$PREVENT_CSR_TO_MANAGE_CUSTOMER_CONNECTORS"
fi

if [ -n "$ENABLE_MSSP_MFA_CHECK" ]; then
  JAVA_OPTS+=" -Dcbc.identity.backend.defense.enable-mssp-mfa-check=$ENABLE_MSSP_MFA_CHECK"
fi

if [ -n "$ENCRYPTED_ONLY_CONNECTOR_LOGIN" ]; then
  JAVA_OPTS+=" -Dcbc.identity.connectorService.encryptedOnlyConnectorLogin=$ENCRYPTED_ONLY_CONNECTOR_LOGIN"
fi

if [ -n "$ENABLE_SOFT_DELETE" ]; then
  JAVA_OPTS+=" -Dcbc.identity.backend.enableSoftDelete=$ENABLE_SOFT_DELETE"
fi

if [ -n "$CONNECTORS_SYNC" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.connectors-sync=$CONNECTORS_SYNC"
fi

if [ "${WAVEFRONT_ENABLED}" == "true" ]; then
    JAVA_OPTS+=" -Dmanagement.metrics.export.wavefront.enabled=true"
    if [ -n "$WAVEFRONT_URI" ]; then
      JAVA_OPTS+=" -Dmanagement.metrics.export.wavefront.uri=$WAVEFRONT_URI"
    fi
    if [ -n "$WAVEFRONT_API_TOKEN" ]; then
      JAVA_OPTS+=" -Dmanagement.metrics.export.wavefront.api-token=$WAVEFRONT_API_TOKEN"
    fi
    if [ -n "$STACK_ID" ]; then
      JAVA_OPTS+=" -Dmetric.tags.stackId=$STACK_ID"
    fi
    if [ -n "$WAVEFRONT_SERVICE_NAME" ]; then
          JAVA_OPTS+=" -Dwavefront.application.service=$WAVEFRONT_SERVICE_NAME"
    fi
else
    JAVA_OPTS+=" -Dmanagement.metrics.export.wavefront.enabled=false"
fi

if [ "${DISABLE_AUTHZ}" == "true" ]; then
  JAVA_OPTS+=" -Dcbc.identity.backend.disable-authorizer=true"
fi

if [ -n "$IDENTITY_CONNECTOR_ID" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.connector_id=$IDENTITY_CONNECTOR_ID"
fi

if [ -n "$DEPRECATE_GET_CONNECTOR_BY_KEY" ]; then
  JAVA_OPTS+=" -Dcbc.identity.backend.defense.deprecate-get-connector-by-key=$DEPRECATE_GET_CONNECTOR_BY_KEY"
fi

if [ -n "$SYNC_SERVICE_URI" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.sync-service-uri=$SYNC_SERVICE_URI"
fi

if [ -n "$SERVICE_ENV" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.env=$SERVICE_ENV"
fi

if [ -n "$SUPPORT_FEATURE_GROUP_ID" ]; then
    JAVA_OPTS+=" -Dcbc.identity.csp.support-feature-group-id=$SUPPORT_FEATURE_GROUP_ID"
fi

if [ -n "$SUPPORT_FEATURE_ROLE" ]; then
    JAVA_OPTS+=" -Dcbc.identity.csp.support-feature-role=$SUPPORT_FEATURE_ROLE"
fi

if [ -n "$SUPPORT_FEATURE_ORG_ID" ]; then
    JAVA_OPTS+=" -Dcbc.identity.csp.support-feature-org-id=$SUPPORT_FEATURE_ORG_ID"
fi

if [ -n "$UPDATE_USER_SESSION" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.update-user-session=$UPDATE_USER_SESSION"
fi

if [ "${ENTITLEMENTS_CHECK}" == "true" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.org-entitlements-check-enabled=true"
else
    JAVA_OPTS+=" -Dcbc.identity.backend.org-entitlements-check-enabled=false"
fi

if [ -n "$JMX_PORT" ]; then
  echo "Enabling JMX connection on port: $JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.port=$JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.rmi.port=$JMX_PORT"
  JAVA_OPTS+=" -Djava.rmi.server.hostname=${JMX_HOST:-127.0.0.1}"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.ssl=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.local.only=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.authenticate=false"
fi

if [ -n "$SAML_KEYSTORE_PASSWORD" ]; then
    JAVA_OPTS+=" -Dsaml.keystore.password=$SAML_KEYSTORE_PASSWORD"
fi

if [ -n "$SAML_KEYSTORE_PATH" ]; then
    JAVA_OPTS+=" -Dsaml.keystore.path=$SAML_KEYSTORE_PATH"
fi

if [ -n "$SAML_CONSUMPTION_URL_BASE" ]; then
    JAVA_OPTS+=" -Dsaml.consumption.url.base=$SAML_CONSUMPTION_URL_BASE"
fi

if [ -n "$SAML_CONSUMPTION_URL_BASE_V2" ]; then
    JAVA_OPTS+=" -Dsaml.consumption.url.base.v2=$SAML_CONSUMPTION_URL_BASE_V2"
fi

if [ -n "$SAML_KEYSTORE_CERT_ALIAS" ]; then
    JAVA_OPTS+=" -Dsaml.keystore.cert.alias=$SAML_KEYSTORE_CERT_ALIAS"
fi


if [ -n "$MULTITENANCY" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.multitenancy-enabled=$MULTITENANCY"
fi

if [ -n "$LOGGER_CONFIG" ]; then
  JAVA_OPTS+=" -Dlog4j2.configurationFile=$LOGGER_CONFIG"
fi

if [ -n "$NOTIFICATIONS" ]; then
  JAVA_OPTS+=" -Dcbc.identity.backend.notifications=$NOTIFICATIONS"
  if [ "${NOTIFICATIONS}" == "redis" ]; then
    JAVA_OPTS+=" -Dspring.redis.ssl=true"
    JAVA_OPTS+=" -Dspring.redis.host=${REDIS_HOST}"
    if [ -n "$REDIS_PORT" ]; then
      JAVA_OPTS+=" -Dspring.redis.port=${REDIS_PORT}"
    fi
    if [ -n "$REDIS_USERNAME" ]; then
      JAVA_OPTS+=" -Dspring.redis.username=${REDIS_USERNAME}"
    fi
    # Don't pass the password directly, expose it as env var
    export SPRING_REDIS_PASSWORD=${REDIS_PASSWORD}
  else
    # Disable redis auto configurations
    JAVA_OPTS+=" -Dspring.autoconfigure.exclude=org.springframework.boot.autoconfigure.data.redis.RedisAutoConfiguration,org.springframework.boot.autoconfigure.data.redis.RedisRepositoriesAutoConfiguration"
  fi
else
  # Disable redis auto configurations
  JAVA_OPTS+=" -Dspring.autoconfigure.exclude=org.springframework.boot.autoconfigure.data.redis.RedisAutoConfiguration,org.springframework.boot.autoconfigure.data.redis.RedisRepositoriesAutoConfiguration"
fi

if [ -n "$EXT_JAVA_OPTS" ]; then
  echo "External java options: $EXT_JAVA_OPTS"
  JAVA_OPTS+=" $EXT_JAVA_OPTS"
fi

# Allow custom props to be inserted
#test -f /custom-conf2.sh && . /custom-conf2.sh

echo "JAVA_OPTS: $JAVA_OPTS"

# Place sensitive JAVA_OPTS below this line
# ================================

if [ -n "$IDENTITY_CONNECTOR_APIKEY" ]; then
    JAVA_OPTS+=" -Dcbc.identity.backend.api-key=$IDENTITY_CONNECTOR_APIKEY"
fi

if [ -n "$DATASOURCE_PASS" ]; then
  JAVA_OPTS+=" -Dspring.datasource.password=$DATASOURCE_PASS"
fi

echo -e '\n\n==============\n\n\n'

##### Start the programs

#touch /tmp/gc_identity-service.txt
#tail -c 0 -F /tmp/gc_identity-service.txt &
#TAILPID=$!

cd "$IDENTITY_ROOT"
java \
    $JAVA_OPTS \
    -cp ${BCFIPS_JARS:+"$BCFIPS_JARS":}:* \
    -jar identity-service.jar \
    "$@" &
PID=$!

EXITCODE=0
wait "$PID" || EXITCODE=$?
echo "Java process exited with $EXITCODE"
#kill "$TAILPID" || true

# Allow custom props to be inserted
#test -f /custom-conf3.sh && . /custom-conf3.sh
#
#if test -e "/rerun.conf" -o -e "/rerun-once.conf"
#then
#  test -x /wavefront-client.sh && (/wavefront-client.sh "jvm.restart.count" "1")
#  echo /rerun*.conf "exists - restarting the java"
#  rm -vf /rerun-once.conf
#  exec "$0" "$@"
#fi

exit "$EXITCODE"
